""" :mod:`telliot_core.types`

The Types package provides mechanisms to specify response types
for Oracle Queries.
"""
# Copyright (c) 2021-, Tellor Development Community
# Distributed under the terms of the MIT License.
